@@ -1,11 +1,12 @@
 //
-// Copyright (C) 2010-2017 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2019 Codership Oy <info@codership.com>
 //
 
 #ifndef GALERA_REPLICATOR_HPP
 #define GALERA_REPLICATOR_HPP
 
 #include "wsrep_api.h"
+#include "wsrep_ps.h"
 #include "galera_exception.hpp"
 #include "trx_handle.hpp"
 
@@ -52,6 +53,7 @@ namespace galera
 
         Replicator() { }
         virtual ~Replicator() { }
+        virtual wsrep_status_t enc_set_key(const wsrep_enc_key_t&) = 0;
         virtual wsrep_status_t connect(const std::string& cluster_name,
                                        const std::string& cluster_url,
                                        const std::string& state_donor,
@@ -112,7 +114,8 @@ namespace galera
         virtual void process_commit_cut(wsrep_seqno_t seq,
                                         wsrep_seqno_t seqno_l) = 0;
         virtual void process_conf_change(void*                    recv_ctx,
-                                         const struct gcs_action& cc) = 0;
+                                         const gcs_act_cchange&   cc,
+                                         const struct gcs_action& act) = 0;
         virtual void process_state_req(void* recv_ctx, const void* req,
                                        size_t req_size,
                                        wsrep_seqno_t seqno_l,
@@ -129,6 +132,18 @@ namespace galera
         // static void stats_free(struct wsrep_stats_var*) must be declared in
         // the child class
 
+        virtual wsrep_status_t fetch_pfs_info(wsrep_node_info_t** nodes,
+                                              uint32_t* size,
+                                              int32_t* my_index,
+                                              uint32_t max_version) = 0;
+        virtual wsrep_status_t fetch_pfs_stat(wsrep_node_stat_t** nodes,
+                                              uint32_t* size,
+                                              int32_t* my_index,
+                                              uint32_t max_version) = 0;
+
+        virtual void free_pfs_info(wsrep_node_info_t* nodes) = 0;
+        virtual void free_pfs_stat(wsrep_node_stat_t* nodes) = 0;
+
         /*! @throws NotFound */
         virtual void        param_set (const std::string& key,
                                        const std::string& value) = 0;
@@ -150,6 +165,13 @@ namespace galera
                                    wsrep_seqno_t seqno_g) = 0;
         virtual bool corrupt() const = 0;
 
+        virtual wsrep_status_t get_membership(wsrep_allocator_cb        alloc,
+                                              struct wsrep_membership** memb)
+            const = 0;
+        virtual wsrep_status_t get_membership(wsrep_allocator_cb        alloc,
+                                              struct wsrep_membership_v2** memb)
+            const = 0;
+
         static void register_params(gu::Config&);
 
     };
