@@ -28,7 +28,7 @@
 #include "gu_atomic.hpp"
 #include "saved_state.hpp"
 #include "gu_debug_sync.hpp"
-
+#include "write_set_wait.hpp"
 
 #include <map>
 #include <queue>
@@ -58,6 +58,11 @@ namespace galera
         int trx_proto_ver() const { return trx_params_.version_; }
         int repl_proto_ver() const{ return protocol_version_; }
 
+        wsrep_status_t enc_set_key(const wsrep_enc_key_t& key)
+        {
+            gcache_.set_enc_key(key); return WSREP_OK;
+        }
+
         wsrep_status_t connect(const std::string& cluster_name,
                                const std::string& cluster_url,
                                const std::string& state_donor,
@@ -117,6 +122,8 @@ namespace galera
         wsrep_status_t replay_trx(TrxHandleMaster& trx,
                                   TrxHandleLock& lock,
                                   void* replay_ctx);
+        wsrep_status_t terminate_trx(TrxHandleMaster& trx,
+                                     wsrep_trx_meta_t* meta);
 
         wsrep_status_t sync_wait(wsrep_gtid_t* upto,
                                  int           tout,
@@ -144,7 +151,9 @@ namespace galera
         void process_trx(void* recv_ctx, const TrxHandleSlavePtr& trx);
         void process_commit_cut(wsrep_seqno_t seq, wsrep_seqno_t seqno_l);
         void submit_view_info(void* recv_ctx, const wsrep_view_info_t* cc);
-        void process_conf_change(void* recv_ctx, const struct gcs_action& cc);
+        void process_conf_change(void*                    recv_ctx,
+                                 const gcs_act_cchange&   cc,
+                                 const struct gcs_action& act);
         void process_state_req(void* recv_ctx, const void* req,
                                size_t req_size, wsrep_seqno_t seqno_l,
                                wsrep_seqno_t donor_seq);
@@ -156,6 +165,18 @@ namespace galera
         void                          stats_reset();
         void                          stats_free(struct wsrep_stats_var*);
 
+        wsrep_status_t fetch_pfs_info(wsrep_node_info_t** nodes,
+                                      uint32_t* size,
+                                      int32_t* my_index,
+                                      uint32_t max_version);
+        wsrep_status_t fetch_pfs_stat(wsrep_node_stat_t** nodes,
+                                      uint32_t* size,
+                                      int32_t* my_index,
+                                      uint32_t max_version);
+
+        void free_pfs_info(wsrep_node_info_t* nodes);
+        void free_pfs_stat(wsrep_node_stat_t* nodes);
+
         /*! @throws NotFound */
         void           set_param (const std::string& key,
                                   const std::string& value);
@@ -182,7 +203,8 @@ namespace galera
         // IST Action handler interface
         void ist_trx(const TrxHandleSlavePtr& ts, bool must_apply,
                      bool preload) override;
-        void ist_cc(const gcs_action&, bool must_apply, bool preload) override;
+        void ist_cc(const gcs_act_cchange&, const gcs_action&, bool must_apply,
+                    bool preload) override;
         void ist_end(const ist::Result&) override;
 
         // Cancel local and enter apply monitors for TrxHandle
@@ -258,8 +280,8 @@ namespace galera
         public:
             ISTEventQueue()
                 :
-                mutex_(),
-                cond_(),
+                mutex_(gu::get_mutex_key(gu::GU_MUTEX_KEY_IST_EVENT_QUEUE)),
+                cond_(gu::get_cond_key(gu::GU_COND_KEY_IST_EVENT_QUEUE)),
                 eof_(false),
                 result_(0, ""),
                 queue_()
@@ -351,6 +373,13 @@ namespace galera
 
         bool corrupt() const { return st_.corrupt(); }
 
+        wsrep_status_t get_membership(wsrep_allocator_cb        alloc,
+                                      struct wsrep_membership** memb) const;
+
+        wsrep_status_t get_membership(wsrep_allocator_cb           alloc,
+                                      struct wsrep_membership_v2** memb) const;
+
+
         struct InitConfig
         {
             InitConfig(gu::Config&, const char* node_addr,const char* base_dir);
@@ -487,6 +516,10 @@ namespace galera
         /* aborts/exits the program in a clean way */
         void abort() GU_NORETURN;
 
+        template <typename M>
+        wsrep_status_t get_membership_tmpl(wsrep_allocator_cb const alloc,
+                                           M**                      memb) const;
+
 #ifdef GALERA_MONITOR_DEBUG_PRINT
     public:
 #endif /* GALERA_MONITOR_DEBUG_PRINT */
@@ -497,27 +530,45 @@ namespace galera
 
             explicit
             LocalOrder(const TrxHandleSlave& ts)
-                :
-                seqno_(ts.local_seqno())
-#if defined(GU_DBUG_ON) || !defined(NDEBUG)
-                ,trx_(&ts)
-#endif //GU_DBUG_ON
+                : seqno_(ts.local_seqno())
+                , cond_(&ts.local_order_cond_)
+                , trx_(&ts)
             { }
 
             LocalOrder(wsrep_seqno_t seqno, const TrxHandleSlave* ts = NULL)
-                :
-                seqno_(seqno)
-#if defined(GU_DBUG_ON) || !defined(NDEBUG)
-                ,trx_(ts)
-#endif //GU_DBUG_ON
+                : seqno_(seqno)
+                , cond_(ts ? &ts->local_order_cond_ :
+                        new gu::Cond(
+                            gu::get_cond_key(gu::GU_COND_KEY_LOCAL_MONITOR)))
+                , trx_(ts)
             {
-#if defined(GU_DBUG_ON) || !defined(NDEBUG)
+                assert(ts || cond_);
                 assert((trx_ && seqno_ == trx_->local_seqno()) || !trx_);
-#endif //GU_DBUG_ON
             }
 
+            ~LocalOrder()
+            {
+                if (not trx_) delete cond_;
+            }
+
+#if defined(GU_DBUG_ON) || !defined(NDEBUG)
+            LocalOrder(const LocalOrder& other)
+                : seqno_(other.seqno_)
+                  // Do not copy cond for debug diagnostics as it may be
+                  // deleted in destructor, which would cause double free.
+                , cond_()
+                , trx_(other.trx_)
+            { }
+
+#else
+            LocalOrder(const LocalOrder&) = delete;
+#endif
+            LocalOrder& operator=(const LocalOrder&) = delete;
+
             wsrep_seqno_t seqno() const { return seqno_; }
 
+            gu::Cond* cond() { return cond_; }
+
             bool condition(wsrep_seqno_t last_entered,
                            wsrep_seqno_t last_left) const
             {
@@ -553,12 +604,10 @@ namespace galera
 
 #ifndef NDEBUG
             LocalOrder()
-                :
-                seqno_(WSREP_SEQNO_UNDEFINED)
-#if defined(GU_DBUG_ON) || !defined(NDEBUG)
-                ,trx_(NULL)
-#endif /* GU_DBUG_ON || !NDEBUG */
-            {}
+                : seqno_(WSREP_SEQNO_UNDEFINED)
+                , cond_()
+                , trx_()
+            { }
 #endif /* NDEBUG */
 
             void print(std::ostream& os) const
@@ -567,15 +616,11 @@ namespace galera
             }
 
         private:
-#ifdef NDEBUG
-            LocalOrder(const LocalOrder& o);
-#endif /* NDEBUG */
             wsrep_seqno_t const seqno_;
-#if defined(GU_DBUG_ON) || !defined(NDEBUG)
-            // this pointer is for debugging purposes only and
-            // is not guaranteed to point at a valid location
+            gu::Cond* cond_;
+            // This pointer is not guaranteed to point into valid location
+            // if the object was copy-constructed.
             const TrxHandleSlave* const trx_;
-#endif /* GU_DBUG_ON || !NDEBUG */
         };
 
         class ApplyOrder
@@ -586,6 +631,7 @@ namespace galera
                 :
                 global_seqno_ (ts.global_seqno()),
                 depends_seqno_(ts.depends_seqno()),
+                cond_(&ts.apply_order_cond_),
                 is_local_     (ts.local()),
                 is_toi_       (ts.is_toi())
 #ifndef NDEBUG
@@ -603,6 +649,7 @@ namespace galera
                 :
                 global_seqno_ (gs),
                 depends_seqno_(ds),
+                cond_(),
                 is_local_     (l),
                 is_toi_       (false)
 #ifndef NDEBUG
@@ -612,6 +659,8 @@ namespace galera
 
             wsrep_seqno_t seqno() const { return global_seqno_; }
 
+            gu::Cond* cond() { return cond_; }
+
             bool condition(wsrep_seqno_t last_entered,
                            wsrep_seqno_t last_left) const
             {
@@ -642,6 +691,7 @@ namespace galera
                 :
                 global_seqno_ (WSREP_SEQNO_UNDEFINED),
                 depends_seqno_(WSREP_SEQNO_UNDEFINED),
+                cond_(),
                 is_local_     (false),
                 is_toi_       (false),
                 trx_          (NULL)
@@ -661,6 +711,7 @@ namespace galera
 #endif /* NDEBUG */
             const wsrep_seqno_t global_seqno_;
             const wsrep_seqno_t depends_seqno_;
+            gu::Cond* cond_;
             const bool is_local_;
             const bool is_toi_;
 #ifndef NDEBUG
@@ -701,6 +752,7 @@ namespace galera
             CommitOrder(const TrxHandleSlave& ts, Mode mode)
                 :
                 global_seqno_(ts.global_seqno()),
+                cond_(&ts.commit_order_cond_),
                 mode_(mode),
                 is_local_(ts.local())
 #ifndef NDEBUG
@@ -711,6 +763,7 @@ namespace galera
             CommitOrder(wsrep_seqno_t gs, Mode mode, bool local = false)
                 :
                 global_seqno_(gs),
+                cond_(),
                 mode_(mode),
                 is_local_(local)
 #ifndef NDEBUG
@@ -720,6 +773,8 @@ namespace galera
 
             wsrep_seqno_t seqno() const { return global_seqno_; }
 
+            gu::Cond* cond() { return cond_; }
+
             bool condition(wsrep_seqno_t last_entered,
                            wsrep_seqno_t last_left) const
             {
@@ -761,6 +816,7 @@ namespace galera
             CommitOrder()
                 :
                 global_seqno_ (WSREP_SEQNO_UNDEFINED),
+                cond_         (),
                 mode_         (OOOC),
                 is_local_     (false),
                 trx_          (NULL)
@@ -780,6 +836,7 @@ namespace galera
             CommitOrder(const CommitOrder&);
 #endif
             const wsrep_seqno_t global_seqno_;
+            gu::Cond* cond_;
             const Mode mode_;
             const bool is_local_;
 #ifndef NDEBUG
@@ -1014,7 +1071,8 @@ namespace galera
         {
         public:
             PendingCertQueue(gcache::GCache& gcache) :
-                mutex_(),
+                mutex_(gu::get_mutex_key(
+                           gu::GU_MUTEX_KEY_PENDING_CERTIFICATION)),
                 ts_queue_(),
                 gcache_(gcache)
             { }
@@ -1073,6 +1131,7 @@ namespace galera
         };
 
         PendingCertQueue pending_cert_queue_;
+        WriteSetWaiters write_set_waiters_;
 
         // concurrency control
         Monitor<LocalOrder>  local_monitor_;
