@@ -1,18 +1,18 @@
 //
-// Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2024 Codership Oy <info@codership.com>
 //
 
 
 #ifndef GALERA_TRX_HANDLE_HPP
 #define GALERA_TRX_HANDLE_HPP
 
-#include "write_set.hpp"
-#include "mapped_buffer.hpp"
 #include "fsm.hpp"
 #include "key_data.hpp" // for append_key()
 #include "key_entry_os.hpp"
 #include "write_set_ng.hpp"
 
+#include "GCache.hpp"
+
 #include "wsrep_api.h"
 #include "gu_mutex.hpp"
 #include "gu_atomic.hpp"
@@ -32,6 +32,7 @@ namespace galera
 {
 
     class NBOCtx; // forward decl
+    class ReplicatorSMM;
 
     static std::string const working_dir = "/tmp";
 
@@ -45,9 +46,7 @@ namespace galera
 
         void add(typename T::State from, typename T::State to)
         {
-            trans_map_.insert_unique(
-                std::make_pair(typename T::Transition(from, to),
-                               typename T::Fsm::TransAttr()));
+            trans_map_.insert_unique(typename T::Transition(from, to));
         }
     private:
         typename T::Fsm::TransMap& trans_map_;
@@ -205,7 +204,6 @@ namespace galera
 
         bool master() const { return master_; }
 
-        void print(std::ostream& os) const;
 
         virtual ~TrxHandle() {}
 
@@ -217,6 +215,8 @@ namespace galera
 
     protected:
 
+        void print(std::ostream& os) const;
+
         void  set_state(State const state, int const line)
         {
             state_.shift_to(state, line);
@@ -359,7 +359,8 @@ namespace galera
     { return ws_flags_to_trx_flags_tmpl<FLAGS_MATCH_API_FLAGS>(flags); }
 
     std::ostream& operator<<(std::ostream& os, TrxHandle::State s);
-    std::ostream& operator<<(std::ostream& os, const TrxHandle& trx);
+    class TrxHandleMaster;
+    std::ostream& operator<<(std::ostream& os, const TrxHandleMaster& trx);
 
     class TrxHandleSlave;
     std::ostream& operator<<(std::ostream& os, const TrxHandleSlave& th);
@@ -417,14 +418,21 @@ namespace galera
             return flags;
         }
 
-        template <bool from_group>
-        size_t unserialize(const gcs_action& act)
+        template <bool from_group, bool decrypt = true>
+        size_t unserialize(gcache::GCache& gcache, const gcs_action& act)
         {
             assert(GCS_ACT_WRITESET == act.type);
 
             try
             {
-                version_ = WriteSetNG::version(act.buf, act.size);
+                void* ptr(const_cast<void*>(act.buf));
+                void* plaintext(decrypt ?
+                                gcache.get_rw_plaintext(ptr) : ptr);
+                /* plaintext resource will be released on GCache::free() later
+                 * in trx lifecycle. In the meantime it will be updated with
+                 * global seqno and certification verdict. */
+
+                version_ = WriteSetNG::version(plaintext, act.size);
                 action_  = std::make_pair(act.buf, act.size);
 
                 switch (version_)
@@ -432,7 +440,8 @@ namespace galera
                 case WriteSetNG::VER3:
                 case WriteSetNG::VER4:
                 case WriteSetNG::VER5:
-                    write_set_.read_buf (act.buf, act.size);
+                case WriteSetNG::VER6:
+                    write_set_.read_buf(plaintext, act.size);
                     assert(version_ == write_set_.version());
                     write_set_flags_ = fixup_write_set_flags(
                         version_,
@@ -639,7 +648,6 @@ namespace galera
 
         void mark_dummy()
         {
-            set_depends_seqno(WSREP_SEQNO_UNDEFINED);
             set_flags(flags() | F_ROLLBACK);
         }
 
@@ -663,6 +671,15 @@ namespace galera
             return !((flags() & F_BEGIN) && (flags() & F_COMMIT));
         }
 
+        // Return true if this is the last fragment of a
+        // streaming replication transaction.
+        bool is_streaming_end() const
+        {
+            return ((write_set_flags_ & F_BEGIN) == 0)
+                   && ((write_set_flags_ & F_COMMIT)
+                       || (write_set_flags_ & F_ROLLBACK));
+        }
+
         void cert_bypass(bool const val)
         {
             assert(true  == val);
@@ -706,6 +723,9 @@ namespace galera
 #ifndef NDEBUG
             ,explicit_rollback_(false)
 #endif /* NDEBUG */
+            , local_order_cond_(gu::get_cond_key(gu::GU_COND_KEY_LOCAL_MONITOR))
+            , apply_order_cond_(gu::get_cond_key(gu::GU_COND_KEY_APPLY_MONITOR))
+            , commit_order_cond_(gu::get_cond_key(gu::GU_COND_KEY_COMMIT_MONITOR))
         {}
 
         friend class TrxHandleMaster;
@@ -733,6 +753,15 @@ namespace galera
         bool                   explicit_rollback_;
 #endif /* NDEBUG */
 
+        // Condition variable for each monitor. Ideally this should be
+        // only one, but we may want to get more specific statistics about
+        // waits per each monitor.
+        friend class ReplicatorSMM;
+        mutable gu::Cond               local_order_cond_;
+        mutable gu::Cond               apply_order_cond_;
+        mutable gu::Cond               commit_order_cond_;
+
+    private:
         TrxHandleSlave(const TrxHandleSlave&);
         void operator=(const TrxHandleSlave& other);
 
@@ -859,14 +888,6 @@ namespace galera
 
         void append_key(const KeyData& key)
         {
-            // Current limitations with certification on trx versions 3 to 5
-            // impose the the following restrictions on keys
-
-            // The shared key behavior for TOI operations is completely
-            // untested, so don't allow it (and it probably does not even
-            // make any sense)
-            assert(is_toi() == false  || key.shared() == false);
-
             /*! protection against protocol change during trx lifetime */
             if (key.proto_ver != version())
             {
@@ -916,7 +937,7 @@ namespace galera
             return write_set_out().gather(source_id(),conn_id(),trx_id(),out);
         }
 
-        void finalize(wsrep_seqno_t const last_seen_seqno)
+        void finalize(wsrep_seqno_t last_seen_seqno)
         {
             assert(last_seen_seqno >= 0);
             assert(ts_ == 0 || last_seen_seqno >= ts_->last_seen_seqno());
@@ -936,7 +957,13 @@ namespace galera
                 }
                 assert(version() >= WriteSetNG::VER5);
                 assert(prev_seqno >= 0);
-                assert(prev_seqno <= last_seen_seqno);
+                // Although commit happens in order, the release of apply
+                // monitor which is used to determine last committed may
+                // not happen in order. Therefore it is possible that the
+                // last seen given as an argument for this method lies
+                // below prev_seqno. Adjust last seen seqno to match
+                // at least prev_seqno which is now known to be committed.
+                last_seen_seqno = std::max(last_seen_seqno, prev_seqno);
                 pa_range = std::min(wsrep_seqno_t(pa_range),
                                     last_seen_seqno - prev_seqno);
             }
@@ -1000,6 +1027,8 @@ namespace galera
         void set_deferred_abort(bool deferred_abort)
         { deferred_abort_ = deferred_abort; }
         bool deferred_abort() const { return deferred_abort_; }
+
+        void print(std::ostream& os) const;
     private:
 
         inline int pa_range_default() const
@@ -1053,7 +1082,7 @@ namespace galera
                         size_t              reserved_size)
             :
             TrxHandle(&trans_map_, source_id, conn_id, trx_id, params.version_),
-            mutex_             (),
+            mutex_             (gu::get_mutex_key(gu::GU_MUTEX_KEY_TRX_HANDLE)),
             mem_pool_          (mp),
             params_            (params),
             ts_                (),
