@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2018-2022 Codership Oy <info@codership.com>
+// Copyright (C) 2018-2023 Codership Oy <info@codership.com>
 //
 
 #include <wsrep_api.h>
@@ -67,6 +67,7 @@ static const char* Defaults[] =
     "gcache.recover",              "yes",
     "gcache.size",                 "128M",
     "gcomm.thread_prio",           "",
+    "gcs.check_appl_proto",        "1",
     "gcs.fc_debug",                "0",
     "gcs.fc_factor",               "1.0",
     "gcs.fc_limit",                "16",
@@ -100,13 +101,12 @@ static const char* Defaults[] =
     "pc.wait_prim",                "true",
     "pc.wait_prim_timeout",        "PT30S",
     "pc.weight",                   "1",
-    "protonet.backend",            "asio",
     "protonet.version",            "0",
     "repl.causal_read_timeout",    "PT30S",
     "repl.commit_order",           "3",
     "repl.key_format",             "FLAT8",
     "repl.max_ws_size",            "2147483647",
-    "repl.proto_max",              "10",
+    "repl.proto_max",              "11",
 #ifdef GU_DBUG_ON
     "signal",                      "",
 #endif
@@ -321,7 +321,6 @@ defaults(bool const enc)
                       ret, strerror(ret));
     }
 
-    provider.free(&provider);
     mark_point();
 
     /* cleanup leftover files */
