@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Codership Oy <info@codership.com>
+/* Copyright (C) 2013-2024 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -19,14 +19,14 @@ class TestKey
 {
 public:
 
-    TestKey (int a, int  ver,
-             bool        exclusive,
-             std::vector<const char*> parts,
-             bool        copy = true)
+    TestKey (int  const ver,
+             wsrep_key_type_t const type,
+             std::vector<const char*> const parts,
+             bool        const copy = true)
         :
         parts_    (),
         ver_      (ver),
-        exclusive_(exclusive),
+        type_     (type),
         copy_     (copy)
     {
         parts_.reserve(parts.size());
@@ -39,24 +39,24 @@ public:
         }
     }
 
-    TestKey (int         ver,
-             bool        exclusive,
-             bool        copy,
-             const char* part0,
-             const char* part1 = 0,
-             const char* part2 = 0,
-             const char* part3 = 0,
-             const char* part4 = 0,
-             const char* part5 = 0,
-             const char* part6 = 0,
-             const char* part7 = 0,
-             const char* part8 = 0,
-             const char* part9 = 0
+    TestKey (int         const ver,
+             wsrep_key_type_t const type,
+             bool        const copy,
+             const char* const part0,
+             const char* const part1 = 0,
+             const char* const part2 = 0,
+             const char* const part3 = 0,
+             const char* const part4 = 0,
+             const char* const part5 = 0,
+             const char* const part6 = 0,
+             const char* const part7 = 0,
+             const char* const part8 = 0,
+             const char* const part9 = 0
         )
         :
         parts_    (),
         ver_      (ver),
-        exclusive_(exclusive),
+        type_     (type),
         copy_     (copy)
     {
         parts_.reserve(10);
@@ -77,9 +77,7 @@ public:
     KeyData
     operator() ()
     {
-        return KeyData (ver_, parts_.data(), parts_.size(),
-                        exclusive_ ? WSREP_KEY_EXCLUSIVE : WSREP_KEY_SHARED,
-                        copy_);
+        return KeyData (ver_, parts_.data(), parts_.size(), type_, copy_);
     }
 
 private:
@@ -87,7 +85,7 @@ private:
     std::vector<wsrep_buf_t> parts_;
 
     int  const ver_;
-    bool const exclusive_;
+    wsrep_key_type_t const type_;
     bool const copy_;
 
     bool
@@ -106,10 +104,4 @@ private:
 
 };
 
-enum
-{
-    SHARED    = false,
-    EXCLUSIVE = true
-};
-
 #endif /* _TEST_KEY_HPP_ */
