@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Codership Oy <info@codership.com> */
+/* Copyright (C) 2013-2016 Codership Oy <info@codership.com> */
 /*!
  * @file Continuous buffer allocator for RecordSet
  *
@@ -15,7 +15,6 @@
 #include "gu_vector.hpp"
 
 #include "gu_macros.h" // gu_likely()
-#include "gu_limits.h" // GU_PAGE_SIZE
 
 #include <cstdlib>     // realloc(), free()
 #include <string>
@@ -42,7 +41,7 @@ public:
 
     explicit
     Allocator (const BaseName&     base_name      = BASE_NAME_DEFAULT,
-               byte_t*             reserved       = NULL,
+               void*               reserved       = NULL,
                page_size_type      reserved_size  = 0,
                heap_size_type      max_heap       = (1U << 22),   /* 4M  */
                page_size_type      disk_page_size = (1U << 26));  /* 64M */
@@ -75,8 +74,8 @@ private:
     {
     public:
 
-        Page (byte_t* ptr, size_t size)
-            : base_ptr_(ptr),
+        Page (void* ptr, size_t size)
+            : base_ptr_(static_cast<gu::byte_t*>(ptr)),
               ptr_     (base_ptr_),
               left_    (size)
         {}
@@ -158,9 +157,6 @@ private:
 
     private:
 
-        /* to avoid too frequent allocation, make it 64K */
-        static page_size_type const PAGE_SIZE = GU_PAGE_SIZE * GU_PAGE_MULTIPLIER;
-
         heap_size_type left_;
 
         Page* my_new_page (page_size_type const size);
