@@ -60,8 +60,7 @@ namespace gu
 
         if (0 == err)
         {
-            unsigned long long free_size = stat.f_bavail;
-            free_size *= stat.f_bsize;
+            unsigned long long  const free_size(stat.f_bavail * stat.f_bsize);
 
             if (reserve < free_size)
             {
@@ -125,7 +124,7 @@ namespace gu
 
             if (ftruncate(fd_, size_))
             {
-                gu_throw_error(errno) << "Failed to truncate '" << name_
+                gu_throw_system_error(errno) << "Failed to truncate '" << name_
                                       << "' to " << size_ << " bytes.";
             }
         }
@@ -139,7 +138,8 @@ namespace gu
     FileDescriptor::constructor_common()
     {
         if (fd_ < 0) {
-            gu_throw_error(errno) << "Failed to open file '" + name_ + '\'';
+            gu_throw_system_error(errno)
+                << "Failed to open file '" + name_ + '\'';
         }
 #if !defined(__APPLE__) /* Darwin does not have posix_fadvise */
 /* benefits are questionable
@@ -181,7 +181,7 @@ namespace gu
         log_debug << "Flushing file '" << name_ << "'";
 
         if (fsync (fd_) < 0) {
-            gu_throw_error(errno) << "fsync() failed on '" + name_ + '\'';
+            gu_throw_system_error(errno) << "fsync() failed on '" + name_ + '\'';
         }
 
         log_debug << "Flushed file '" << name_ << "'";
@@ -193,10 +193,12 @@ namespace gu
         byte_t const byte (0);
 
         if (lseek (fd_, offset, SEEK_SET) != offset)
-            gu_throw_error(errno) << "lseek() failed on '" << name_ << '\'';
+            gu_throw_system_error(errno)
+                << "lseek() failed on '" << name_ << '\'';
 
         if (write (fd_, &byte, sizeof(byte)) != sizeof(byte))
-            gu_throw_error(errno) << "write() failed on '" << name_ << '\'';
+            gu_throw_system_error(errno)
+                << "write() failed on '" << name_ << '\'';
 
         return true;
     }
@@ -222,7 +224,7 @@ namespace gu
             return;
         }
 
-        gu_throw_error (errno) << "File preallocation failed";
+        gu_throw_system_error (errno) << "File preallocation failed";
     }
 
     void
@@ -234,19 +236,22 @@ namespace gu
                   << name_ << "'...";
 
 #if defined(__APPLE__)
-        if (0 != fcntl (fd_, F_SETSIZE, size_) && 0 != ftruncate (fd_, size_))
-#else
-        if (0 != posix_fallocate (fd_, start, diff))
-#endif
+        if (-1 == fcntl (fd_, F_SETSIZE, size_) && -1 == ftruncate (fd_, size_))
         {
-            if ((EINVAL == errno || ENOSYS == errno) && start >= 0 && diff > 0)
+#else
+        int const ret = posix_fallocate (fd_, start, diff);
+        if (0 != ret)
+        {
+            errno = ret;
+#endif
+            if ((EINVAL == errno || ENOSYS == errno || EOPNOTSUPP == errno) && start >= 0 && diff > 0)
             {
                 // FS does not support the operation, try physical write
                 write_file (start);
             }
             else
             {
-                gu_throw_error (errno) << "File preallocation failed";
+                gu_throw_system_error (errno) << "File preallocation failed";
             }
         }
     }
