@@ -98,12 +98,25 @@ public:
         }
     }
 
+    virtual void
+    update_address_info(const std::string& local_addr,
+                        const std::string& remote_addr) GALERA_OVERRIDE
+    {
+        ck_assert_int_eq(address_info_updated, false);
+        address_info_updated = true;
+    }
+
+    virtual void update_SSL_info() GALERA_OVERRIDE {
+        ck_assert(address_info_updated);
+    }
+
     enum op_status next_result;
     int next_error;
     size_t count_client_handshake_called;
     size_t count_server_handshake_called;
     size_t count_read_called;
     size_t count_write_called;
+    bool address_info_updated{false};
 
 private:
     int fd_;
@@ -208,6 +221,7 @@ public:
     }
     size_t bytes_written() const { return bytes_written_; }
     const gu::AsioErrorCode& last_error_code() const { return last_error_code_; }
+
 private:
     std::array<std::string, 2> write_buffer_;
     std::string read_buffer_;
@@ -1126,6 +1140,9 @@ static EVP_PKEY* create_key()
     BN_free(bn);
     return pkey;
 #else
+#if OPENSSL_VERSION_NUMBER < 0x30004000L
+    OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS, NULL);
+#endif
     auto* ret = EVP_RSA_gen(2048);
     if (!ret)
     {
@@ -1152,6 +1169,12 @@ static void write_key(EVP_PKEY* pkey, const std::string& filename)
     const std::string cert_dir = get_cert_dir();
     const std::string key_file_path = cert_dir + "/" + filename;
     auto* key_file = open_file(key_file_path, "wb");
+    if (!key_file)
+    {
+        const auto* errstr = ::strerror(errno);
+        gu_throw_fatal << "Could not open file " << key_file_path << ": "
+                       << errstr;
+    }
     if (!PEM_write_PrivateKey(key_file, pkey, nullptr, nullptr, 0, password_cb,
                               nullptr))
     {
@@ -1248,6 +1271,12 @@ static void write_x509(X509* x509, const std::string& filename)
     const std::string cert_dir = get_cert_dir();
     const std::string file_path = cert_dir + "/" + filename;
     auto* file = open_file(file_path, "wb");
+    if (!file)
+    {
+        const auto* errstr = ::strerror(errno);
+        gu_throw_fatal << "Could not open file " << file_path << ": "
+                       << errstr;
+    }
     if (!PEM_write_X509(file, x509))
     {
         throw_error("Could not write x509");
@@ -1261,6 +1290,12 @@ static void write_x509_list(const std::vector<X509*>& certs,
     const std::string cert_dir = get_cert_dir();
     const std::string file_path = cert_dir + "/" + filename;
     auto* file = open_file(file_path, "wb");
+    if (!file)
+    {
+        const auto* errstr = ::strerror(errno);
+        gu_throw_fatal << "Could not open file " << file_path << ": "
+                       << errstr;
+    }
     for (auto* x509 : certs)
     {
         if (!PEM_write_X509(file, x509))
