@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2021 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2024 Codership Oy <info@codership.com>
  */
 
 #include "gcache_rb_store.hpp"
@@ -28,10 +28,13 @@ namespace gcache
     {
         write_preamble(false);
 
-        for (seqno2ptr_iter_t i = seqno2ptr_.begin(); i != seqno2ptr_.end(); ++i)
+        for (seqno2ptr_iter_t i = seqno2ptr_.begin(); i != seqno2ptr_.end();)
         {
             if (ptr2BH(*i)->ctx == BH_ctx_t(this)) {
-                seqno2ptr_.erase(i);
+                i = seqno2ptr_.erase(i);
+            }
+            else {
+                ++i;
             }
         }
 
@@ -70,6 +73,7 @@ namespace gcache
         next_      (first_),
         seqno2ptr_ (seqno2ptr),
         gid_       (gid),
+        seqno_locked_(SEQNO_MAX),
         size_cache_(end_ - start_ - sizeof(BufferHeader)),
         size_free_ (size_cache_),
         size_used_ (0),
@@ -118,7 +122,7 @@ namespace gcache
 
             BufferHeader* const bh(ptr2BH(*j));
 
-            if (gu_likely (BH_is_released(bh)))
+            if (gu_likely (BH_is_released(bh) && bh->seqno_g < seqno_locked_))
             {
                 seqno2ptr_.erase (j);
 
