@@ -5,12 +5,12 @@
 #ifndef __GCACHE_SEQNO__
 #define __GCACHE_SEQNO__
 
-#include <stdint.h>
+#include <wsrep_api.h>
 #include <limits>
 
 namespace gcache
 {
-    typedef int64_t seqno_t;
+    typedef wsrep_seqno_t seqno_t;
 
     static seqno_t const SEQNO_NONE =  0;
     static seqno_t const SEQNO_ILL  = -1;
