@@ -41,6 +41,7 @@ namespace gcomm
         typedef std::vector<InputMapNode> InputMapNodeIndex;
         std::ostream& operator<<(std::ostream&, const InputMapNodeIndex&);
         class InputMap;
+        std::ostream& operator<<(std::ostream&, const InputMap&);
     }
 }
 
@@ -321,9 +322,19 @@ public:
     iterator recover(const size_t uuid, const seqno_t seq) const;
 
     /*!
+     * Return list of ranges for missing messages.
      *
+     * @param index Index of the node.
+     * @param range Range to be scanned for missing messages.
      */
-    void reset(const size_t, const seqno_t = 256);
+    std::vector<Range> gap_range_list(size_t index, const Range& range) const;
+
+    /*!
+     * Reset the input map.
+     *
+     * @param nodes Number of nodes in the new configuration.
+     */
+    void reset(const size_t nodes);
 
     /*!
      * Clear input map state.
@@ -347,12 +358,11 @@ private:
      */
     void cleanup_recovery_index();
 
-    seqno_t            window_;
-    seqno_t            safe_seq_;       /*!< Safe seqno              */
-    seqno_t            aru_seq_;        /*!< All received upto seqno */
-    InputMapNodeIndex* node_index_;     /*!< Index of nodes          */
-    InputMapMsgIndex*  msg_index_;      /*!< Index of messages       */
-    InputMapMsgIndex*  recovery_index_; /*!< Recovery index          */
+    seqno_t            safe_seq_;       /*!< Safe seqno               */
+    seqno_t            aru_seq_;        /*!< All received up to seqno */
+    InputMapNodeIndex* node_index_;     /*!< Index of nodes           */
+    InputMapMsgIndex*  msg_index_;      /*!< Index of messages        */
+    InputMapMsgIndex*  recovery_index_; /*!< Recovery index           */
 };
 
 #endif // EVS_INPUT_MAP2_HPP
