@@ -103,7 +103,8 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
     bind_ip_      (""),
     mcast_ttl_    (check_range(
                        Conf::GMCastMCastTTL,
-                       param<int>(conf_, uri, Conf::GMCastMCastTTL, "1"),
+                       param<int>(conf_, uri, Conf::GMCastMCastTTL,
+                                  Defaults::GMCastMCastTTL),
                        1, 256)),
     listener_     (),
     mcast_        (),
