@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2025 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -11,17 +11,28 @@
 #ifndef _gcs_group_h_
 #define _gcs_group_h_
 
-#include <stdbool.h>
-
+#include <wsrep_ps.h> // pfs declarations
 #include "gcs_gcache.hpp"
 #include "gcs_node.hpp"
 #include "gcs_recv_msg.hpp"
 #include "gcs_seqno.hpp"
 #include "gcs_state_msg.hpp"
+#include "gu_unordered.hpp"
+#include "gu_mutex.hpp"
+#include "gu_thread_keys.hpp"
+
+#include "gu_config.hpp"
+
+extern std::string const GCS_STATELESS_KEY;
+extern std::string const GCS_VOTE_POLICY_KEY;
+extern uint8_t gcs_group_conf_to_vote_policy(gu::Config& cnf);
 
 #include "gu_status.hpp"
 #include "gu_utils.hpp"
 
+#include <wsrep_membership_service.h>
+#include <stdbool.h>
+
 typedef enum gcs_group_state
 {
     GCS_GROUP_NON_PRIMARY,
@@ -35,9 +46,19 @@ gcs_group_state_t;
 
 extern const char* gcs_group_state_str[];
 
+typedef gu::UnorderedMap<gu::GTID, int64_t, gu::GTID::TableHash> VoteHistory;
+
+struct VoteResult { gcs_seqno_t seqno; int64_t res; };
+
 typedef struct gcs_group
 {
+    mutable
+    gu::Mutex     memb_mtx_;    // syncs updates by the recv thread and reads
+                                // by gcs_group_get_membership()
+    gcs_seqno_t   memb_epoch_;
+
     gcache_t*     cache;
+    gu::Config&   cnf;
     gcs_seqno_t   act_id_;      // current(last) action seqno
     gcs_seqno_t   conf_id;      // current configuration seqno
     gu_uuid_t     state_uuid;   // state exchange id
@@ -48,8 +69,13 @@ typedef struct gcs_group
     const char*   my_address;
     gcs_group_state_t state;    // group state: PRIMARY | NON_PRIMARY
     gcs_seqno_t   last_applied; // last_applied action group-wide
-    long          last_node;    // node that reported last_applied
+    long          last_node;    // node that last reported commit_cut
+    gcs_seqno_t   vote_request_seqno; // last vote request was passed for it
+    VoteResult    vote_result;  // last vote result
+    VoteHistory   vote_history; // history of group votes
+    uint8_t       vote_policy;
     bool          frag_reset;   // indicate that fragmentation was reset
+    bool          stateless;
     gcs_node_t*   nodes;        // array of node contexts
 
     /* values from the last primary component */
@@ -69,30 +95,34 @@ typedef struct gcs_group
     gcs_state_quorum_t quorum;
     int last_applied_proto_ver;
 
-    gcs_group() : gcs_proto_ver(0), repl_proto_ver(0), appl_proto_ver(0) { }
+    gcs_group(gu::Config&  cnf,
+              gcache_t*    cache,
+              const char*  node_name, ///< can be null
+              const char*  inc_addr,  ///< can be null
+              gcs_proto_t  gcs_proto_ver,
+              int          repl_proto_ver,
+              int          appl_proto_ver);
+    ~gcs_group();
 
+    void
+    get_protocols(int& appl, int& repl, int& gcs)
+    {
+        appl = quorum.appl_proto_ver;
+        repl = quorum.repl_proto_ver;
+        gcs  = quorum.gcs_proto_ver;
+    }
+
+    static void
+    register_params(gu::Config& cnf);
 }
 gcs_group_t;
 
-/*!
- * Initialize group at startup
- */
-extern int
-gcs_group_init (gcs_group_t* group,
-                gcache_t*    cache,
-                const char*  node_name, ///< can be null
-                const char*  inc_addr,  ///< can be null
-                gcs_proto_t  gcs_proto_ver,
-                int          repl_proto_ver,
-                int          appl_proto_ver);
-
 /*!
  * Initialize group action history parameters. See gcs.h
  */
 extern int
-gcs_group_init_history (gcs_group_t*     group,
-                        gcs_seqno_t      seqno,
-                        const gu_uuid_t* uuid);
+gcs_group_init_history (gcs_group_t*    group,
+                        const gu::GTID& position);
 
 #ifdef GCS_CORE_TESTING
 /*!
@@ -103,12 +133,6 @@ extern void
 group_nodes_free (gcs_group_t* group);
 #endif // GCS_CORE_TESTING
 
-/*!
- * Free group resources
- */
-extern void
-gcs_group_free (gcs_group_t* group);
-
 /*! Forget the action if it is not to be delivered */
 extern void
 gcs_group_ignore_action (gcs_group_t* group, struct gcs_act_rcvd* rcvd);
@@ -133,6 +157,9 @@ gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg);
 extern gcs_seqno_t
 gcs_group_handle_last_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg);
 
+extern VoteResult
+gcs_group_handle_vote_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg);
+
 /*! @return 0 for success, 1 for (success && i_am_sender)
  * or negative error code */
 extern int
@@ -159,7 +186,7 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
                           struct gcs_act_rcvd*  const rcvd,
                           bool commonly_supported_version)
 {
-    long const sender_idx = msg->sender_idx;
+    int  const sender_idx = msg->sender_idx;
     bool const local      = (sender_idx == group->my_idx);
     ssize_t ret;
 
@@ -167,6 +194,7 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
     assert (sender_idx < group->num);
     assert (frg->act_id > 0);
     assert (frg->act_size > 0);
+    assert (gcs_act_in_cache(frg->act_type));
 
     // clear reset flag if set by own first fragment after reset flag was set
     group->frag_reset = (group->frag_reset &&
@@ -181,8 +209,9 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
         assert (ret == rcvd->act.buf_len);
 
         rcvd->act.type = frg->act_type;
+        rcvd->sender_idx = sender_idx;
 
-        if (gu_likely(GCS_ACT_TORDERED  == rcvd->act.type &&
+        if (gu_likely(GCS_ACT_WRITESET  == rcvd->act.type &&
                       GCS_GROUP_PRIMARY == group->state   &&
                       group->nodes[sender_idx].status >= GCS_NODE_STATE_DONOR &&
                       !(group->frag_reset && local) &&
@@ -192,12 +221,12 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
              * and only in PRIM (skip messages while in state exchange) */
             rcvd->id = ++group->act_id_;
         }
-        else if (GCS_ACT_TORDERED  == rcvd->act.type) {
+        else if (GCS_ACT_WRITESET  == rcvd->act.type) {
             /* Rare situations */
             if (local) {
                 /* Let the sender know that it failed */
                 rcvd->id = -ERESTART;
-                gu_debug("Returning -ERESTART for TORDERED action: group->state"
+                gu_debug("Returning -ERESTART for WRITESET action: group->state"
                          " = %s, sender->status = %s, frag_reset = %s, "
                          "buf = %p",
                          gcs_group_state_str[group->state],
@@ -236,11 +265,11 @@ gcs_group_my_idx (const gcs_group_t* group)
 /*!
  * Creates new configuration action
  * @param group group handle
- * @param act   GCS action object
+ * @param rcvd  GCS action object
  * @param proto protocol version gcs should use for this configuration
  */
 extern ssize_t
-gcs_group_act_conf (gcs_group_t* group, struct gcs_act* act, int* proto);
+gcs_group_act_conf (gcs_group_t* group, struct gcs_act_rcvd* rcvd, int* proto);
 
 /*! Returns state object for state message */
 extern gcs_state_msg_t*
@@ -258,9 +287,36 @@ gcs_group_find_donor(const gcs_group_t* group,
                      int const str_version,
                      int const joiner_idx,
                      const char* const donor_string, int const donor_len,
-                     const gu_uuid_t* ist_uuid, gcs_seqno_t ist_seqno);
+                     const gu::GTID& ist_gtid);
 
-extern void
+extern int
+gcs_group_param_set(gcs_group_t& group,
+                    const std::string& key, const std::string& val);
+
+extern int
 gcs_group_get_status(const gcs_group_t* group, gu::Status& status);
 
+void
+gcs_group_get_membership(const gcs_group_t& group,
+                         wsrep_allocator_cb alloc,
+                         struct wsrep_membership** memb);
+void
+gcs_group_get_membership(const gcs_group_t& group,
+                         wsrep_allocator_cb alloc,
+                         struct wsrep_membership_v2** memb);
+
+extern int
+gcs_group_fetch_pfs_info(const gcs_group_t*  group,
+                         wsrep_node_info_t** nodes,
+                         uint32_t*           size,
+                         int32_t*            my_index,
+                         uint32_t            max_version);
+
+extern int
+gcs_group_fetch_pfs_stat(const gcs_group_t*  group,
+                         wsrep_node_stat_t** nodes,
+                         uint32_t*           size,
+                         int32_t*            my_index,
+                         uint32_t            max_version);
+
 #endif /* _gcs_group_h_ */
