@@ -1,35 +1,53 @@
 /*
- * Copyright (C) 2008 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
  *
  * $Id$
  */
 
-#include <check.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-
-#include "gcs_defrag_test.hpp"
 #include "../gcs_defrag.hpp"
 
+#include "gcs_test_utils.hpp"
+#include <gcache_test_encryption.hpp>
+
+#include "gcs_defrag_test.hpp" // must be included last
+
 #define TRUE (0 == 0)
 #define FALSE (!TRUE)
 
 static void
 defrag_check_init (gcs_defrag_t* defrag)
 {
-    fail_if (defrag->sent_id  != GCS_SEQNO_ILL);
-    fail_if (defrag->head     != NULL);
-    fail_if (defrag->tail     != NULL);
-    fail_if (defrag->size     != 0);
-    fail_if (defrag->received != 0);
-    fail_if (defrag->frag_no  != 0);
+    ck_assert(defrag->sent_id  == GCS_SEQNO_ILL);
+    ck_assert(defrag->head     == NULL);
+    ck_assert(defrag->tail     == NULL);
+    ck_assert(defrag->size     == 0);
+    ck_assert(defrag->received == 0);
+    ck_assert(defrag->frag_no  == 0);
 }
 
-START_TEST (gcs_defrag_test)
+static void
+defrag(bool const enc)
 {
     ssize_t ret;
 
+    // Environment
+    gu::Config config;
+    std::string const cache_name("defrag.cache");
+    gcs_test::InitConfig(config, cache_name);
+    gcache::GCache* cache;
+
+    if (enc)
+    {
+        cache = new gcache::GCache(NULL, config, ".", gcache_test_encrypt_cb,
+                                   NULL);
+        wsrep_enc_key_t const key = { cache, sizeof(*cache) };
+        cache->set_enc_key(key);
+    }
+    else
+    {
+        cache = new gcache::GCache(NULL, config, ".", NULL, NULL);
+    }
+
     // The Action
     char         act_buf[]  = "Test action smuction";
     size_t       act_len    = sizeof (act_buf);
@@ -54,18 +72,13 @@ START_TEST (gcs_defrag_test)
 
     mark_point();
 
-#ifndef NDEBUG
-    // debug build breaks this test due to asserts
-    return;
-#endif
-
     // Initialize message parameters
     frg1.act_id    = getpid();
     frg1.act_size  = act_len;
     frg1.frag      = frag1;
     frg1.frag_len  = frag1_len;
     frg1.frag_no   = 0;
-    frg1.act_type  = GCS_ACT_TORDERED;
+    frg1.act_type  = GCS_ACT_WRITESET;
     frg1.proto_ver = 0;
 
     // normal fragments
@@ -87,31 +100,32 @@ START_TEST (gcs_defrag_test)
     mark_point();
 
     // ready for the first fragment
-    gcs_defrag_init (&defrag, NULL);
+    gcs_defrag_init (&defrag, reinterpret_cast<gcache_t*>(cache));
     defrag_check_init (&defrag);
 
     mark_point();
 
     // 1. Try fragment that is not the first
     ret = gcs_defrag_handle_frag (&defrag, &frg3, &recv_act, FALSE);
-    fail_if (ret != -EPROTO);
+    ck_assert(ret == -EPROTO);
     mark_point();
     defrag_check_init (&defrag); // should be no changes
 
     // 2. Try first fragment
     ret = gcs_defrag_handle_frag (&defrag, &frg1, &recv_act, FALSE);
-    fail_if (ret != 0);
-    fail_if (defrag.head == NULL);
-    fail_if (defrag.received != frag1_len);
-    fail_if (defrag.tail != defrag.head + defrag.received);
+    ck_assert(ret == 0);
+    ck_assert(defrag.head != NULL);
+    ck_assert(defrag.received == frag1_len);
+    ck_assert(defrag.tail ==
+              static_cast<uint8_t*>(defrag.plain) + defrag.received);
     tail = defrag.tail;
 
 #define TRY_WRONG_2ND_FRAGMENT(frag)                                    \
     ret = gcs_defrag_handle_frag (&defrag, &frag, &recv_act, FALSE);    \
-    if (defrag.frag_no < frag.frag_no) fail_if (ret != -EPROTO);        \
-    else fail_if (ret != 0);                                            \
-    fail_if (defrag.received != frag1_len);                             \
-    fail_if (defrag.tail != tail);
+    if (defrag.frag_no < frag.frag_no) ck_assert(ret == -EPROTO);       \
+    else ck_assert(ret == 0);                                           \
+    ck_assert(defrag.received == frag1_len);                            \
+    ck_assert(defrag.tail == tail);
 
     // 3. Try first fragment again
     TRY_WRONG_2ND_FRAGMENT(frg1);
@@ -124,45 +138,62 @@ START_TEST (gcs_defrag_test)
 
     // 6. Try second fragment
     ret = gcs_defrag_handle_frag (&defrag, &frg2, &recv_act, FALSE);
-    fail_if (ret != 0);
-    fail_if (defrag.received != frag1_len + frag2_len);
-    fail_if (defrag.tail != defrag.head + defrag.received);
+    ck_assert(ret == 0);
+    ck_assert(defrag.received == frag1_len + frag2_len);
+    ck_assert(defrag.tail ==
+              static_cast<uint8_t*>(defrag.plain) + defrag.received);
 
     // 7. Try third fragment, last one
     ret = gcs_defrag_handle_frag (&defrag, &frg3, &recv_act, FALSE);
-    fail_if (ret != (long)act_len);
+    ck_assert(ret == (long)act_len);
+
+#define CHECK_ACTION(origin)                                            \
+    {                                                                   \
+        const char* ptx                                                 \
+            (static_cast<const char*>(cache->get_ro_plaintext(recv_act.buf))); \
+        ck_assert_msg(!strncmp(ptx, act_buf, act_len),                  \
+                      "%s action received: '%s', expected '%s'",        \
+                      origin, ptx, act_buf);                            \
+        cache->free(const_cast<void*>(recv_act.buf));                   \
+    }
 
     // 8. Check the action
-    fail_if (recv_act.buf == NULL);
-    fail_if (recv_act.buf_len != (long)act_len);
-    fail_if (strncmp((const char*)recv_act.buf, act_buf, act_len),
-             "Action received: '%s', expected '%s'",recv_act.buf,act_buf);
+    ck_assert(recv_act.buf != NULL);
+    ck_assert(recv_act.buf_len == (long)act_len);
+    CHECK_ACTION("Remote");
     defrag_check_init (&defrag); // should be empty
 
-    gcs_gcache_free(defrag.cache, recv_act.buf);
-
     // 9. Try the same with local action
     ret = gcs_defrag_handle_frag (&defrag, &frg1, &recv_act, TRUE);
-    fail_if (ret != 0);
-//    fail_if (defrag.head != NULL); (and now we may allocate it for cache)
+    ck_assert(ret == 0);
 
     ret = gcs_defrag_handle_frag (&defrag, &frg2, &recv_act, TRUE);
-    fail_if (ret != 0);
-//    fail_if (defrag.head != NULL); (and now we may allocate it for cache)
+    ck_assert(ret == 0);
 
     ret = gcs_defrag_handle_frag (&defrag, &frg3, &recv_act, TRUE);
-    fail_if (ret != (long)act_len);
-//    fail_if (defrag.head != NULL); (and now we may allocate it for cache)
-
+    ck_assert(ret == (long)act_len);
+    CHECK_ACTION("Local");
 
     // 10. Check the action
-    fail_if (recv_act.buf == NULL);
-    fail_if (recv_act.buf_len != (long)act_len);
-//    fail_if (recv_act.buf != NULL); (and now we may allocate it for cache)
+    ck_assert(recv_act.buf != NULL);
+    ck_assert(recv_act.buf_len == (long)act_len);
 
     defrag_check_init (&defrag); // should be empty
 
-    gcs_gcache_free(defrag.cache, recv_act.buf);
+    mark_point();
+    delete cache;
+    ::unlink(cache_name.c_str());
+}
+
+START_TEST (gcs_defrag_test)
+{
+    defrag(false);
+}
+END_TEST
+
+START_TEST (gcs_defrag_testE)
+{
+    defrag(true);
 }
 END_TEST
 
@@ -173,6 +204,7 @@ Suite *gcs_defrag_suite(void)
 
   suite_add_tcase (suite, tcase);
   tcase_add_test  (tcase, gcs_defrag_test);
+  tcase_add_test  (tcase, gcs_defrag_testE);
   return suite;
 }
 
